﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using CountrLight.Core.Models;
using CountrLight.Core.Services;
using CountrLight.Core.ViewModels;
using GalaSoft.MvvmLight.Messaging;
using GalaSoft.MvvmLight.Views;
using Moq;
using NUnit.Framework;

namespace CountrLight.Core.Tests.ViewModels
{
    [TestFixture]
    public class CountersViewModelTests
    {
        Mock<ICountersService> countersService;
        Mock<IMessenger> messenger;
        Mock<INavigationService> navigationService;
        Action<CountersChangedMessage> publishAction;
        CountersViewModel viewModel;

        [SetUp]
        public void SetUp()
        {
            countersService = new Mock<ICountersService>();
            messenger = new Mock<IMessenger>();
            messenger.Setup(m => m.Register(It.IsAny<CountersViewModel>(),
                                            It.IsAny<Action<CountersChangedMessage>>()))
                     .Callback<object, Action<CountersChangedMessage>>((o, a) => publishAction = a);
            navigationService = new Mock<INavigationService>();
            ViewModelLocator.RegisterDispatcher(a => a());

            viewModel = new CountersViewModel(countersService.Object, messenger.Object, navigationService.Object);
        }

        [Test]
        public async Task LoadCounters_CreatesCounters()
        {
            // Arrange
            var counters = new List<Counter>
            {
                new Counter{Name = "Counter1", Count=0},
                new Counter{Name = "Counter2", Count=4},
            };
            countersService.Setup(c => c.GetAllCounters())
                           .ReturnsAsync(counters);
            // Act
            await viewModel.LoadCounters();
            // Assert
            Assert.AreEqual(2, viewModel.Counters.Count);
            Assert.AreEqual("Counter1", viewModel.Counters[0].Name);
            Assert.AreEqual(0, viewModel.Counters[0].Count);
            Assert.AreEqual("Counter2", viewModel.Counters[1].Name);
            Assert.AreEqual(4, viewModel.Counters[1].Count);
        }

        [Test]
        public void ReceivedMessage_LoadsCounters()
        {
            // Arrange
            countersService.Setup(s => s.GetAllCounters()).ReturnsAsync(new List<Counter>());
            // Act
            publishAction.Invoke(new CountersChangedMessage());
            // Assert
            countersService.Verify(s => s.GetAllCounters());
        }

        [Test]
        public void ShowAddNewCounterCommand_ShowsCounterViewModel()
        {
            // Act                                                             
            viewModel.ShowAddNewCounterCommand.Execute(null);
            // Assert                                                          
            navigationService.Verify(n => n.NavigateTo(nameof(CounterViewModel),
                                                       It.IsAny<Counter>()));
        }
    }
}